@@ -48,6 +48,7 @@ from sdbus import (
     dbus_property_async,
     dbus_property_async_override,
     dbus_signal_async,
+    get_current_message,
 )
 
 
@@ -98,6 +99,11 @@ class TestInterface(DbusInterfaceCommonAsync,
         """Uppercase the input"""
         return string.upper()
 
+    @dbus_method_async(result_signature='s')
+    async def get_sender(self) -> str:
+        message = get_current_message()
+        return message.sender or ''
+
     @dbus_method_async(result_signature='x')
     async def test_int(self) -> int:
         return 1
@@ -201,12 +207,15 @@ class DbusErrorUnmappedLater(DbusFailedError):
     dbus_error_name = 'org.example.Nothing'
 
 
+TEST_SERVICE_NAME = 'org.example.test'
+
+
 def initialize_object() -> Tuple[TestInterface, TestInterface]:
     test_object = TestInterface()
     test_object.export_to_dbus('/')
 
     test_object_connection = TestInterface.new_proxy(
-        "org.example.test", '/')
+        TEST_SERVICE_NAME, '/')
 
     return test_object, test_object_connection
 
@@ -214,7 +223,7 @@ def initialize_object() -> Tuple[TestInterface, TestInterface]:
 class TestProxy(IsolatedDbusTestCase):
     async def asyncSetUp(self) -> None:
         await super().asyncSetUp()
-        await self.bus.request_name_async("org.example.test", 0)
+        await self.bus.request_name_async(TEST_SERVICE_NAME, 0)
 
     async def test_method_kwargs(self) -> None:
         test_object, test_object_connection = initialize_object()
@@ -278,12 +287,14 @@ class TestProxy(IsolatedDbusTestCase):
                 test_object_connection.test_struct_return_workaround(), 0.5),
         )
 
+        self.assertTrue(await test_object_connection.get_sender())
+
     async def test_subclass(self) -> None:
         test_object, test_object_connection = initialize_object()
 
         test_var = ['asdasd']
 
-        class TestInheritnce(TestInterface):
+        class TestInheritence(TestInterface):
             @dbus_method_async_override()
             async def test_int(self) -> int:
                 return 2
@@ -297,14 +308,14 @@ class TestProxy(IsolatedDbusTestCase):
                 nonlocal test_var
                 test_var.insert(0, var)
 
-        test_subclass = TestInheritnce()
+        test_subclass = TestInheritence()
 
         test_subclass.export_to_dbus('/subclass', self.bus)
 
         self.assertEqual(await test_subclass.test_int(), 2)
 
-        test_subclass_connection = TestInheritnce.new_proxy(
-            "org.example.test", '/subclass', self.bus)
+        test_subclass_connection = TestInheritence.new_proxy(
+            TEST_SERVICE_NAME, '/subclass', self.bus)
 
         self.assertEqual(await test_subclass_connection.test_int(), 2)
 
@@ -315,13 +326,33 @@ class TestProxy(IsolatedDbusTestCase):
         self.assertEqual(test_var[0], await test_subclass.test_property)
         self.assertEqual('12345', await test_subclass.test_property)
 
+        with self.subTest('Test dbus to python mapping'):
+            self.assertIn(
+                test_object.properties_changed.dbus_signal.signal_name,
+                test_object._dbus_to_python_name_map,
+            )
+
+            self.assertIn(
+                test_subclass.properties_changed.dbus_signal.signal_name,
+                test_subclass._dbus_to_python_name_map,
+            )
+
+            self.assertIn(
+                test_subclass.test_property.dbus_property.property_name,
+                test_subclass._dbus_to_python_name_map,
+            )
+
     async def test_bad_subclass(self) -> None:
-        def bad_call() -> None:
-            class TestInheritnce(TestInterface):
+        with self.assertRaises(TypeError):
+            class TestInheritence(TestInterface):
                 async def test_int(self) -> int:
                     return 2
 
-        self.assertRaises(TypeError, bad_call)
+        with self.assertRaises(TypeError):
+            class TestInheritence2(TestInterface):
+                @dbus_method_async_override()
+                async def test_unrelated(self) -> int:
+                    return 2
 
     async def test_properties(self) -> None:
         test_object, test_object_connection = initialize_object()
@@ -378,46 +409,92 @@ class TestProxy(IsolatedDbusTestCase):
 
         self.assertEqual(test_tuple, await wait_for(q.get(), timeout=1))
 
+    async def test_signal_catch_anywhere(self) -> None:
+        test_object, test_object_connection = initialize_object()
+
+        loop = get_running_loop()
+
+        test_tuple = ('sgfsretg', 'asd')
+
+        with self.subTest('Catch anywhere over D-Bus object'):
+            async def catch_anywhere_oneshot_dbus(
+            ) -> Tuple[str, Tuple[str, str]]:
+                async for x in test_object_connection.test_signal\
+                        .catch_anywhere():
+                    return x
+
+                raise RuntimeError
+
+            catch_anywhere_over_dbus_task \
+                = loop.create_task(catch_anywhere_oneshot_dbus())
+
+            await sleep(0)
+
+            test_object.test_signal.emit(test_tuple)
+
+            self.assertEqual(
+                ('/', test_tuple),
+                await wait_for(catch_anywhere_over_dbus_task, timeout=1),
+            )
+
+        with self.subTest('Catch anywhere over D-Bus class'):
+            async def catch_anywhere_oneshot_from_class(
+            ) -> Tuple[str, Tuple[str, str]]:
+                async for x in TestInterface.test_signal.catch_anywhere(
+                        TEST_SERVICE_NAME, self.bus):
+                    return x
+
+                raise RuntimeError
+
+            catch_anywhere_from_class_task \
+                = loop.create_task(catch_anywhere_oneshot_from_class())
+
+            await sleep(0)
+
+            test_object.test_signal.emit(test_tuple)
+
+            self.assertEqual(
+                ('/', test_tuple),
+                await wait_for(catch_anywhere_from_class_task, timeout=1),
+            )
+
+        with self.subTest('Catch anywhere over local object'):
+            async def catch_anywhere_oneshot_local(
+            ) -> Tuple[str, Tuple[str, str]]:
+                async for x in test_object.test_signal.catch_anywhere():
+                    return x
+
+                raise RuntimeError
+
+            catch_anywhere_over_local_task \
+                = loop.create_task(catch_anywhere_oneshot_local())
+
+            with self.assertRaises(NotImplementedError):
+                await wait_for(
+                    catch_anywhere_over_local_task,
+                    timeout=1,
+                )
+
     async def test_exceptions(self) -> None:
         test_object, test_object_connection = initialize_object()
 
         async def first_test() -> None:
             await test_object_connection.raise_base_exception()
 
-        loop = get_running_loop()
-
-        t1 = loop.create_task(first_test())
-
-        try:
-            await wait_for(t1, timeout=1)
-        except DbusFailedError:
-            ...
-
-        self.assertRaises(DbusFailedError, t1.result)
+        with self.assertRaises(DbusFailedError):
+            await wait_for(first_test(), timeout=1)
 
         async def second_test() -> None:
             await test_object_connection.raise_derived_exception()
 
-        t2 = loop.create_task(second_test())
-
-        try:
-            await wait_for(t2, timeout=1)
-        except DbusFileExistsError:
-            ...
-
-        self.assertRaises(DbusFileExistsError, t2.result)
+        with self.assertRaises(DbusFileExistsError):
+            await wait_for(second_test(), timeout=1)
 
         async def third_test() -> None:
             await test_object_connection.raise_custom_error()
 
-        t3 = loop.create_task(third_test())
-
-        try:
-            await wait_for(t3, timeout=1)
-        except DbusFailedError:
-            ...
-
-        self.assertRaises(DbusErrorTest, t3.result)
+        with self.assertRaises(DbusErrorTest):
+            await wait_for(third_test(), timeout=1)
 
         def bad_exception_error_name_used() -> None:
             class BadDbusError(DbusFailedError):
@@ -434,14 +511,8 @@ class TestProxy(IsolatedDbusTestCase):
         async def test_unmapped_expection() -> None:
             await test_object_connection.raise_and_unmap_error()
 
-        t4 = loop.create_task(test_unmapped_expection())
-
-        try:
-            await wait_for(t4, timeout=1)
-        except SdBusUnmappedMessageError:
-            ...
-
-        self.assertRaises(SdBusUnmappedMessageError, t4.result)
+        with self.assertRaises(SdBusUnmappedMessageError):
+            await wait_for(test_unmapped_expection(), timeout=1)
 
     async def test_no_reply_method(self) -> None:
         test_object, test_object_connection = initialize_object()
@@ -460,16 +531,9 @@ class TestProxy(IsolatedDbusTestCase):
 
         collect()
 
-        loop = get_running_loop()
-
-        t = loop.create_task(test_object_connection.dbus_introspect())
-
-        try:
-            await wait_for(t, timeout=0.2)
-        except DbusUnknownObjectError:
-            ...
-
-        self.assertRaises(DbusUnknownObjectError, t.result)
+        with self.assertRaises(DbusUnknownObjectError):
+            await wait_for(test_object_connection.dbus_introspect(),
+                           timeout=0.2)
 
     def test_docstring(self) -> None:
         test_object, test_object_connection = initialize_object()
@@ -498,7 +562,7 @@ class TestProxy(IsolatedDbusTestCase):
         loop = get_running_loop()
 
         async def catch_property_emit_connection() -> str:
-            async for x in test_object_connection.properties_changed:
+            async for x in test_object_connection.properties_changed.catch():
                 for v in x[1].values():
                     probably_str = v[1]
                     if isinstance(probably_str, str):
@@ -508,7 +572,7 @@ class TestProxy(IsolatedDbusTestCase):
             raise ValueError
 
         async def catch_property_emit_local() -> str:
-            async for x in test_object.properties_changed:
+            async for x in test_object.properties_changed.catch():
                 for v in x[1].values():
                     probably_str = v[1]
                     if isinstance(probably_str, str):
@@ -576,27 +640,19 @@ class TestProxy(IsolatedDbusTestCase):
     async def test_bus_close(self) -> None:
         test_object, test_object_connection = initialize_object()
 
-        loop = get_running_loop()
-
         async def too_long_wait() -> None:
             await test_object_connection.looong_method()
 
-        t1 = loop.create_task(too_long_wait())
-
         self.bus.close()
 
-        try:
-            await wait_for(t1, timeout=1)
-        except SdBusLibraryError:
-            ...
-
-        self.assertRaises(SdBusLibraryError, t1.result)
+        with self.assertRaises(SdBusLibraryError):
+            await wait_for(too_long_wait(), timeout=1)
 
     async def test_singal_queue_wildcard_match(self) -> None:
         test_object, test_object_connection = initialize_object()
 
         message_queue = await self.bus.get_signal_queue_async(
-            'org.example.test',
+            TEST_SERVICE_NAME,
             None, None, None)
 
         test_object.test_signal.emit(('test', 'signal'))
@@ -703,3 +759,21 @@ class TestProxy(IsolatedDbusTestCase):
             '^Invalid signal name',
             test_bad_signal_name,
         )
+
+    async def test_properties_get_all_dict(self) -> None:
+        test_object, test_object_connection = initialize_object()
+
+        dbus_dict = await test_object_connection._properties_get_all(
+            'org.test.test')
+
+        self.assertEqual(
+            await test_object.test_property,
+            dbus_dict['TestProperty'][1],
+        )
+
+        self.assertEqual(
+            await test_object.test_property,
+            (
+                await test_object_connection.properties_get_all_dict()
+            )['test_property'],
+        )
